#!/usr/bin/perl -w

package ALTLinux::ACL;
use strict;
use warnings;
use File::Path qw(make_path);
use File::Basename;
use IPC::Run3;

our $VERSION = 0.11;
our $PKGNAME2ACL;
our $GRPNAME2ACL;
my $aclpackageprefix='list.packages.';
my $aclgroupprefix='list.groups.';
my @cachedirs=grep {$_} ($ENV{HOME}.'/.cache/altlinux-acl','.');

our $default_expire = 5/24; # -M; 5/24=5 hours
our $default_branch = 'sisyphus';
our $default_verbose = 1;

our @LONGOPT = (
    'acl-branch=s' => \$default_branch,
    'acl-expire=f' => \$default_expire,
    'acl-verbose=i' => \$default_verbose,
    );

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-expire=> $default_expire, # -M; 5/24=5 hours
	-branch=> $default_branch,
	-verbose=> $default_verbose,
	@_,
    };
    bless($self,$class);
    unless ($PKGNAME2ACL) {
	$PKGNAME2ACL=$self->__load(-packages,$aclpackageprefix);
	$GRPNAME2ACL=$self->__load(-groups,$aclgroupprefix);
	# save from expansion
	$GRPNAME2ACL->{'@nobody'}=['@nobody'];
	$GRPNAME2ACL->{'@everybody'}=['@everybody'];
    }
    return $self;
}

sub acl {
    my ($self,$item)=@_;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return @$ptr;
}

sub is_authorized {
    my ($self,$item, $user)=@_;
    my %expanded = map {$_=>1} $self->acl_expanded($item);
    return $expanded{$user} || $expanded{'@everybody'} || $expanded{'@nobody'};
}

sub is_leader {
    my ($self,$item, $user)=@_;
    my $raw_leader=$PKGNAME2ACL->{$item}->[0];
    return 1 if $raw_leader eq $user;
    return 0 if substr($raw_leader, 0, 1) ne '@';
    # indirect leader
    $raw_leader=$GRPNAME2ACL->{$raw_leader}->[0];
    return $raw_leader eq $user;
}

sub leader {
    my ($self,$item)=@_;
    my $raw_leader=$PKGNAME2ACL->{$item}->[0];
    $raw_leader=$GRPNAME2ACL->{$item}->[0] unless $raw_leader;
    return $raw_leader;
}

sub pkglist($) {
    my ($self)=@_;
    return keys(%$PKGNAME2ACL);
}

sub grplist($) {
    my ($self)=@_;
    return keys(%$GRPNAME2ACL);
}

############ PUBLIC STATIC ###############
# i.e. called ALTLinux::ACL::<sub name>

sub email2aclname {
    my ($email)=@_;
    $email=~/^.+\<(\S+)\s*(?:\@|at)\s*altlinux.*>/;
    my $user=$1;
    unless ($user) {
	warn "e-mail [$email] does not look like ALT Linux maintainer e-mail.";
	return;
    }
    # for php-coder (acl name php_coder).
    $user=~s/-/_/g;
    return $user;
}

########## PROTECTED #################

# I'm not sure how to name it. If you use it, e-mail me.
sub acl_expanded {
    my ($self,$item)=@_;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return map {$GRPNAME2ACL->{$_} ? @{$GRPNAME2ACL->{$_}} : $_ } @$ptr;
}

########## PRIVATE #################

sub __load {
    my ($self, $aclopt, $aclprefix)=@_;
    my $cache={};
    # let us tolerate 'Sisyphus', etc, to...
    my $branch=lc($self->{'-branch'});
    my $aclfile=$self->{$aclopt};
    if (not defined $aclfile) {
	my @acllist = grep {defined($_) and -e $_} $self->{$aclopt}, map {$_.'/'.$aclprefix.$branch} @cachedirs;
	$aclfile||=$acllist[0];
    }
    $aclfile||=$cachedirs[0].'/'.$aclprefix.$branch;

    # $aclfile contains creation timestamp, useful for wget speedup.
    # this is check timestamp.
    my $timestampfile=$aclfile.'.timestamp';
    if (not -e $aclfile or not -e $timestampfile or -M $timestampfile > $self->{-expire}) {
	make_path(dirname($aclfile));
	my $url='http://git.altlinux.org/acl/'.$aclprefix.$branch;
	print STDERR "updating acl cache: wget -q -O $aclfile $url\n" if $self->{-verbose};
	my $wget_stderr;
	run3 ['wget', '-q', '-O'.$aclfile, $url], \undef, \undef, \$wget_stderr;
	my $retval=$?;
	unless ($retval==0) {
	    unlink $aclfile unless -e $aclfile and -s $aclfile;
	    warn "WARNING: can't update acl: wget -q -O$aclfile $url failed ($retval): $wget_stderr\n";
	    die "can't download acl file $url to $aclfile\n" unless -e $aclfile;
	}
	die "can't find acl file $aclfile" unless -e $aclfile;
	system ('touch', $timestampfile);
    }

    open (ACL,'<',$aclfile) || die "can't open acl file $aclfile: $!";
    while (<ACL>) {
	chomp;
	if (/([^\t]+)\t(.+)/) {
	    my $name=$1;
	    my @acl=split (/\s+/,$2);
	    $cache->{$name}=\@acl if @acl;
	} else {
	    warn "acl: strange string $_";
	}
    }
    close(ACL);
    $self->{$aclopt}=$aclfile;
    return $cache;
}

1;

=head1	NAME

ALTLinux::ACL - ALTLinux ACL query library.

=head1	DESCRIPTION

ALTLinux ACL query library automatically downloads, caches, and
provides an object interface to ALTLinux acl files.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2010 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

